<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

class ProcessamentoSolicitacaoValeCombustivel extends ProcessamentoSolicitacao {
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrbeneficiom          = $itens['NRBENEFICIOM']   ?? null;
        $dsobsbeneficiario     = $itens['DSOBSBENEFICIARIO']   ?? null;
        $cdTipoaltevc          = $itens['CDTIPOALTEVC']   ?? null;
        $nrpessoa              = $itens['NRPESSOA'] ?? null;
        $idtpbeneficiario      = "T";
        $qtbeneficio           = $itens['QTBENEFICIO'] ?? null;
        $dtinscbeneficiario    = isset($itens['DTINSCBENEFICIARIO']) ? DateUtil::getDataDeString(substr($itens['DTINSCBENEFICIARIO'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtcancbeneficiario    = isset($itens['DTCANCBENEFICIARIO']) ? DateUtil::getDataDeString(substr($itens['DTCANCBENEFICIARIO'],0,10), DateUtil::FORMATO_BRASILEIRO, true) : null;
        $dtatual               = DateUtil::getDataDeString(substr(date('d/m/Y'),0,10), DateUtil::FORMATO_BRASILEIRO, true);
        $nrorg                 = Utils::getParamters()->getNrorg();
        $nrvinculom            = $this->getSolicitacao()->getNrvinculom();

        if($cdTipoaltevc === 'E') {
            $this->removeVabBeneficiario($nrorg, $nrvinculom, $nrbeneficiom);
            return;
        }

        $beneficiario = Utils::getEntityManager()->getRepository(Repositories::VAB_BENEFICIARIO)->findBy(array("nrorg"=> $nrorg, "nrvinculom" => $nrvinculom, "nrbeneficiom" => $nrbeneficiom));

        if(isset($beneficiario[0])) {
            throw new \Exception("Beneficiário já cadastrado.");
        }
        
        $vabBeneficiario = Repositories::VAB_BENEFICIARIO;
        $vabBeneficiario = new $vabBeneficiario;
        $vabBeneficiario->setNrorg($nrorg);
        $vabBeneficiario->setNrvinculom($nrvinculom);
        $vabBeneficiario->setNrpessoa($nrpessoa);
        $vabBeneficiario->setNrbeneficiom($nrbeneficiom);
        $vabBeneficiario->setDtinscbeneficiario($dtinscbeneficiario);
        $vabBeneficiario->setDtcancbeneficiario($dtcancbeneficiario);
        $vabBeneficiario->setDsobsbeneficiario($dsobsbeneficiario);
        $vabBeneficiario->setQtbeneficio($qtbeneficio);
        $vabBeneficiario->setIdtpbeneficiario($idtpbeneficiario);
        $vabBeneficiario->setDtinclusao(DateUtil::getDataAtual(false));
        $vabBeneficiario->setDtultatu($dtatual);
        $vabBeneficiario->setIdvalordifebenef(null);
        $vabBeneficiario->setNrorginclusao(Utils::getParamters()->getNrorg());
        $vabBeneficiario->setNrorgultatu(Utils::getParamters()->getNrorg());
        $vabBeneficiario->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $vabBeneficiario->setCdoperultatu(Utils::getParamters()->getCdoperador());
        $vabBeneficiario->setNrbeneficiario(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'VAB_BENEFICIARIO', 1, 12, true));
        Utils::getEntityManager()->persist($vabBeneficiario);
        Utils::getEntityManager()->flush();
    }
    
    public function rejeitar() {}

    public function removeVabBeneficiario($nrorg, $nrvinculom, $nrbeneficiom) {
        $sql = <<<SQL
        DELETE FROM VAB_BENEFICIARIO VB
        WHERE VB.NRORG = :P_NRORG
        AND VB.NRVINCULOM = :P_NRVINCULOM
        AND VB.NRBENEFICIOM = :P_NRBENEFICIOM
SQL;

        return Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                "P_NRORG"        => $nrorg,
                "P_NRVINCULOM"   => $nrvinculom,
                "P_NRBENEFICIOM" => $nrbeneficiom
            )
        );
    }
}
