<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Ems\Repository\ControleEntregaEPIRepository;

class ControleEntregaEPIService {
    /** @var ControleEntregaEPIRepository */
    public $controleEntregaEPIVinculoRepository;

    /** @var Environment */
    public $environment;

    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        VinculoService $vinculoService,
        ControleEntregaEPIRepository $controleEntregaEPIRepository
    ) {
        $this->entityManager                = $entityManager;
        $this->vinculoService               = $vinculoService;
        $this->controleEntregaEPIRepository = $controleEntregaEPIRepository;
        $this->environment                  = $environment;
    }

    public function getControleEntregaEPI($nrestruturam, $nrperiodoapuracao) : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();
        $dtmescompetenc = $this->environment->getDtmescomptencDiaCorrente();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg,
            'P_NRESTRUTURAM'    => $nrestruturam,
            'P_DTMESCOMPETENC'  => $dtmescompetenc
        ];
        
        $controleEntregaEPI = $this->controleEntregaEPIRepository->getControleEntregaEPI($params);;

        return $controleEntregaEPI;
    }

    public function getEpi() : array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
            'P_NRORG'           => $nrOrg,
        ];

        $epi = $this->controleEntregaEPIRepository->getEpi($params);;

        return $epi;
    }

    public function setHistoricoEpi($row) {
        try {
            $historicoEpiSaved = false;
            
            $isRowNew = $row['__is_new'] ?? false;

            // Salva os Históricos
            if($isRowNew) {
                $historicoEpiSaved = $this->newHistoricoEpi($row);
            } else {
                $historicoEpiSaved = $this->updateHistoricoEpi($row);
            }

            return $historicoEpiSaved;
		} catch (\Exception $e) {
			throw $e;
		}
    }

    public function newHistoricoEpi($row) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $success = $this->controleEntregaEPIRepository->newHistoricoEpi($row);

        return $success;
    }
    
    public function updateHistoricoEpi($row) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $success = $this->controleEntregaEPIRepository->updateHistoricoEpi($row);

        return $success;
    }
    
    public function deleteHistoricoEpi($row) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $success = $this->controleEntregaEPIRepository->deleteHistoricoEpi($row);

        return $success;
    }
}