<?php

namespace Tests\IntegrationTests;

use Tests\Util;
use PHPUnit\Framework\TestCase;

abstract class HcmServicesIntegrationTestCase extends TestCase {
    abstract public static function createTestData(): void;
    abstract public static function destroyTestData(): void;

    public function setUp(): void {
        Util::setDefaultTestSession();
    }

    public static function setUpBeforeClass(): void {
        Util::setDefaultTestSession();
        self::beforeDataManipulation();
        static::destroyTestData();
        static::createTestData();
        self::afterDataManipulation();
    }

    public static function tearDownAfterClass(): void {
        self::beforeDataManipulation();
        static::destroyTestData();
        Util::updateAllNovoCodigoTables();
        self::afterDataManipulation();
    }

    /**
     * Aciona rotinas antes da manipulacao dos dados de testes
     * @return void
     */
    private static function beforeDataManipulation(): void {
        Util::beginTransactionOnConnection();
        Util::disableAutoCommitOnConnection();
    }

    /**
     * Aciona rotinas apos da manipulacao dos dados de testes
     * @return void
     */
    private static function afterDataManipulation(): void {
        Util::commitOnConnection();
        Util::enableAutoCommitOnConnection();
    }
}
