<?php

namespace Tests\IntegrationTests;

use Tests\Util;

class IntegracaoEstruturaTest extends HcmServicesIntegrationTestCase {

    /**
     * Caso de Testes 1
     */
    public function testBuscaEstruturasLegais() {
        $apiService = Util::getInstanceManager()->getService('apiService');

        $estruturasEncontradas = $apiService->getEstruturaPorTipo(
            null, 1
        );

        $nrEstruturasEsperadas = [
            Util::DEFAULT_PK, Util::DEFAULT_PK-2
        ];

        $asEstruturasEsperadasForamBuscadas = array_reduce($nrEstruturasEsperadas,
            function ($carry, $nrEstruturaEsperada) use ($estruturasEncontradas) {
                return $carry && count(array_filter($estruturasEncontradas, function ($estruturaEncontrada) use ($nrEstruturaEsperada) {
                    return $estruturaEncontrada['NRESTRUTURAM'] == $nrEstruturaEsperada;
                })) == 1;
            }, true
        );

        $this->assertTrue($asEstruturasEsperadasForamBuscadas);
    }

    /**
     * Caso de Testes 2
     */
    public function testBuscaEstruturasGerenciais() {
        $apiService = Util::getInstanceManager()->getService('apiService');

        $estruturasEncontradas = $apiService->getEstruturaPorTipo(
            null, 2
        );

        $nrEstruturasEsperadas = [
            Util::DEFAULT_PK-1
        ];

        $asEstruturasEsperadasForamBuscadas = array_reduce($nrEstruturasEsperadas,
            function ($carry, $nrEstruturaEsperada) use ($estruturasEncontradas) {
                return $carry && count(array_filter($estruturasEncontradas, function ($estruturaEncontrada) use ($nrEstruturaEsperada) {
                    return $estruturaEncontrada['NRESTRUTURAM'] == $nrEstruturaEsperada;
                })) == 1;
            }, true
        );
        $this->assertTrue($asEstruturasEsperadasForamBuscadas);
    }

    public static function createTestData(): void {
        $DEFAULT_PK = Util::DEFAULT_PK;
        $DEFAULT_ORG = Util::DEFAULT_ORG;

        $sqls = [
            // Parceiro de negocio estrutura 1
            "Insert into PARCNEGOCIO (NRPARCNEGOCIO,NRORG,CDSEGMER,NMPRINCIPALPARC,NMSECUNDARIPARC,DTNASCIFUNDPARC,NRINSCRICAOPARC,IDATIVO,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,CDTIPOPARCPRINCIPAL,CDTIPOINSCRICAO,IDPESSOAFISICA,IDINSTITUICAO,DSDEPARTAMENTO,IDIDIOMA,CDATIVCLI,NRSEQTPATIVCLI,IDPARCFUNDIDO,CDOPERRESPONSAVEL,NRTPCLASSIFICAPARC,DSDIRLOGOMARCA,CDINSCESTA,CDINSCMUNI)
            values ({$DEFAULT_PK},{$DEFAULT_ORG},null,'ESTRUTURA TESTES INTEGRACAO 1','ESTRUTURA TESTES INTEGRACAO 1',to_date('01/01/2000 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,'S',to_date('19/03/2020 15:55:13','DD/MM/RRRR HH24:MI:SS'),to_date('20/03/2020 17:30:15','DD/MM/RRRR HH24:MI:SS'),{$DEFAULT_ORG},null,{$DEFAULT_ORG},null,'ESTRUTURA','LIVRE','S','N',null,null,null,null,'N',null,null,null,null,null)",

            // Estrutura 1
            "Insert into ESTRUTURAM (NRESTRUTURAM,NRORG,NRTIPOESTRUTURA,DTINIVIGENCIA,DTFIMVIGENCIA,CDINTESTRUTURA,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,NRESTRUTURAMOLD,NRPARCNEGOCIO,NMESTRUTURAM,DSESTRUTURA,IDATIVO,CDINTESTRUTEXP,CDINTESTRUTEXPFIL,CDCENTCUST,DSURLLOGOESTRUTURA,NRTPEVENTOCTBL,NRREGIMEAPUR,DTESOCIAL,NRGRUPOESOCIAL,CDEXPPDRATEIO)
            values ({$DEFAULT_PK},{$DEFAULT_ORG},(Select NRTPESTRUTLEGAL From GPE_PARAMORG Where NRORG = {$DEFAULT_ORG}),to_date('01/01/2000 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,null,to_date('01/01/2000 10:03:00','DD/MM/RRRR HH24:MI:SS'),null,null,null,null,null,null,{$DEFAULT_PK},'ESTRUTURA TESTES INTEGRACAO 1',null,'S',null,null,null,null,null,null,null,null,null)",

            // Historico Estrutura 1
            "Insert into ESTRUTURAH (NRESTRUTURAH,NRORG,NRESTRUTURAM,DTMESCOMPETENC,DTEXTINCAO,NRCALENDARIO,CDCNPJESTRUT,CDCEIESTRUT,NMRAZSOCESTRUT,CDNATUJURI,CDCNAE,NRCAGED,CDEMPRCAIXA,IDTIPOEMPR,IDOPTSIMPLES,IDPARTICIPAT,NRPARCNEGOCIO,NRPROPRIEMPR,NRPARCRESPINF,NRPARCENTISIN,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,CDFPAS,CDREDUCONT,NMFANTASIA,NMESTRUTURAH,CDCPFESTRUTURA,DTBASESINDICAL,CDSINDICAL,CDTERCEIRO,NRCLASSTRIBUT,VRFAP,NRTIPOTRIBUT,VRALIQDESON,CDFEDERALESTRUT,IDATIVO,NRPARCREGAMB,NRPARCMONBIO,DTINIRESAMB,DTFIMRESAMB,NRTPOBRIGAPRENDIZ,NRTPOBRIGAPCD,CDCAEPFESTRUT,CODCONVENIO,NRCDPROCESSORAT,VRPERCENTRAT,NRCDPROCESSOFAP,VRPERCENTFAP,NRCDCONVENIOTERCEIRO,NRPROCESSOTERCEIRO,NRCDCONTRATAPRENDIZ,IDENTIDSFIMLUCRATIVO,NRCDPROCESSOCONTRATPCD,NRPARCRESPINFADIC,IDENVFAPESO)
            values ({$DEFAULT_PK},{$DEFAULT_ORG},{$DEFAULT_PK},to_date('01/01/2023 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,{$DEFAULT_PK},null,null,'ESTRUTURA TESTES INTEGRACAO 1',null,null,null,null,null,'N','N',{$DEFAULT_PK},null,null,null,to_date('01/01/2023 10:53:53','DD/MM/RRRR HH24:MI:SS'),null,null,${DEFAULT_ORG},null,null,null,null,'ESTRUTURA TESTES INTEGRACAO','ESTRUTURA TESTES INTEGRACAO',null,null,null,null,null,null,null,null,null,'S',null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,'N')",

            // Parceiro de negocio estrutura 2
            "Insert into PARCNEGOCIO (NRPARCNEGOCIO,NRORG,CDSEGMER,NMPRINCIPALPARC,NMSECUNDARIPARC,DTNASCIFUNDPARC,NRINSCRICAOPARC,IDATIVO,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,CDTIPOPARCPRINCIPAL,CDTIPOINSCRICAO,IDPESSOAFISICA,IDINSTITUICAO,DSDEPARTAMENTO,IDIDIOMA,CDATIVCLI,NRSEQTPATIVCLI,IDPARCFUNDIDO,CDOPERRESPONSAVEL,NRTPCLASSIFICAPARC,DSDIRLOGOMARCA,CDINSCESTA,CDINSCMUNI)
            values ({$DEFAULT_PK}-1,{$DEFAULT_ORG},null,'ESTRUTURA TESTES INTEGRACAO 2','ESTRUTURA TESTES INTEGRACAO 2',to_date('01/01/2000 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,'S',to_date('19/03/2020 15:55:13','DD/MM/RRRR HH24:MI:SS'),to_date('20/03/2020 17:30:15','DD/MM/RRRR HH24:MI:SS'),{$DEFAULT_ORG},null,{$DEFAULT_ORG},null,'ESTRUTURA','LIVRE','S','N',null,null,null,null,'N',null,null,null,null,null)",

            // Estrutura 2
            "Insert into ESTRUTURAM (NRESTRUTURAM,NRORG,NRTIPOESTRUTURA,DTINIVIGENCIA,DTFIMVIGENCIA,CDINTESTRUTURA,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,NRESTRUTURAMOLD,NRPARCNEGOCIO,NMESTRUTURAM,DSESTRUTURA,IDATIVO,CDINTESTRUTEXP,CDINTESTRUTEXPFIL,CDCENTCUST,DSURLLOGOESTRUTURA,NRTPEVENTOCTBL,NRREGIMEAPUR,DTESOCIAL,NRGRUPOESOCIAL,CDEXPPDRATEIO)
            values ({$DEFAULT_PK}-1,{$DEFAULT_ORG},(Select NRTPESTRUTGEREN From GPE_PARAMORG Where NRORG = {$DEFAULT_ORG}),to_date('01/01/2000 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,null,to_date('01/01/2000 10:03:00','DD/MM/RRRR HH24:MI:SS'),null,null,null,null,null,null,{$DEFAULT_PK}-1,'ESTRUTURA TESTES INTEGRACAO 2',null,'S',null,null,null,null,null,null,null,null,null)",

            // Historico Estrutura 2
            "Insert into ESTRUTURAH (NRESTRUTURAH,NRORG,NRESTRUTURAM,DTMESCOMPETENC,DTEXTINCAO,NRCALENDARIO,CDCNPJESTRUT,CDCEIESTRUT,NMRAZSOCESTRUT,CDNATUJURI,CDCNAE,NRCAGED,CDEMPRCAIXA,IDTIPOEMPR,IDOPTSIMPLES,IDPARTICIPAT,NRPARCNEGOCIO,NRPROPRIEMPR,NRPARCRESPINF,NRPARCENTISIN,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,CDFPAS,CDREDUCONT,NMFANTASIA,NMESTRUTURAH,CDCPFESTRUTURA,DTBASESINDICAL,CDSINDICAL,CDTERCEIRO,NRCLASSTRIBUT,VRFAP,NRTIPOTRIBUT,VRALIQDESON,CDFEDERALESTRUT,IDATIVO,NRPARCREGAMB,NRPARCMONBIO,DTINIRESAMB,DTFIMRESAMB,NRTPOBRIGAPRENDIZ,NRTPOBRIGAPCD,CDCAEPFESTRUT,CODCONVENIO,NRCDPROCESSORAT,VRPERCENTRAT,NRCDPROCESSOFAP,VRPERCENTFAP,NRCDCONVENIOTERCEIRO,NRPROCESSOTERCEIRO,NRCDCONTRATAPRENDIZ,IDENTIDSFIMLUCRATIVO,NRCDPROCESSOCONTRATPCD,NRPARCRESPINFADIC,IDENVFAPESO)
            values ({$DEFAULT_PK}-1,{$DEFAULT_ORG},{$DEFAULT_PK}-1,to_date('01/01/2023 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,{$DEFAULT_PK},null,null,'ESTRUTURA TESTES INTEGRACAO 2',null,null,null,null,null,'N','N',{$DEFAULT_PK},null,null,null,to_date('01/01/2023 10:53:53','DD/MM/RRRR HH24:MI:SS'),null,null,${DEFAULT_ORG},null,null,null,null,'ESTRUTURA TESTES INTEGRACAO','ESTRUTURA TESTES INTEGRACAO',null,null,null,null,null,null,null,null,null,'S',null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,'N')",

            // Parceiro de negocio estrutura 3
            "Insert into PARCNEGOCIO (NRPARCNEGOCIO,NRORG,CDSEGMER,NMPRINCIPALPARC,NMSECUNDARIPARC,DTNASCIFUNDPARC,NRINSCRICAOPARC,IDATIVO,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,CDTIPOPARCPRINCIPAL,CDTIPOINSCRICAO,IDPESSOAFISICA,IDINSTITUICAO,DSDEPARTAMENTO,IDIDIOMA,CDATIVCLI,NRSEQTPATIVCLI,IDPARCFUNDIDO,CDOPERRESPONSAVEL,NRTPCLASSIFICAPARC,DSDIRLOGOMARCA,CDINSCESTA,CDINSCMUNI)
            values ({$DEFAULT_PK}-2,{$DEFAULT_ORG},null,'ESTRUTURA TESTES INTEGRACAO 3','ESTRUTURA TESTES INTEGRACAO 3',to_date('01/01/2000 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,'S',to_date('19/03/2020 15:55:13','DD/MM/RRRR HH24:MI:SS'),to_date('20/03/2020 17:30:15','DD/MM/RRRR HH24:MI:SS'),{$DEFAULT_ORG},null,{$DEFAULT_ORG},null,'ESTRUTURA','LIVRE','S','N',null,null,null,null,'N',null,null,null,null,null)",

            // Estrutura 3
            "Insert into ESTRUTURAM (NRESTRUTURAM,NRORG,NRTIPOESTRUTURA,DTINIVIGENCIA,DTFIMVIGENCIA,CDINTESTRUTURA,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,NRESTRUTURAMOLD,NRPARCNEGOCIO,NMESTRUTURAM,DSESTRUTURA,IDATIVO,CDINTESTRUTEXP,CDINTESTRUTEXPFIL,CDCENTCUST,DSURLLOGOESTRUTURA,NRTPEVENTOCTBL,NRREGIMEAPUR,DTESOCIAL,NRGRUPOESOCIAL,CDEXPPDRATEIO)
            values ({$DEFAULT_PK}-2,{$DEFAULT_ORG},(Select NRTPESTRUTLEGAL From GPE_PARAMORG Where NRORG = {$DEFAULT_ORG}),to_date('01/01/2000 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,null,to_date('01/01/2000 10:03:00','DD/MM/RRRR HH24:MI:SS'),null,null,null,null,null,null,{$DEFAULT_PK}-2,'ESTRUTURA TESTES INTEGRACAO 3',null,'S',null,null,null,null,null,null,null,null,null)",

            // Historico Estrutura 3
            "Insert into ESTRUTURAH (NRESTRUTURAH,NRORG,NRESTRUTURAM,DTMESCOMPETENC,DTEXTINCAO,NRCALENDARIO,CDCNPJESTRUT,CDCEIESTRUT,NMRAZSOCESTRUT,CDNATUJURI,CDCNAE,NRCAGED,CDEMPRCAIXA,IDTIPOEMPR,IDOPTSIMPLES,IDPARTICIPAT,NRPARCNEGOCIO,NRPROPRIEMPR,NRPARCRESPINF,NRPARCENTISIN,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,CDFPAS,CDREDUCONT,NMFANTASIA,NMESTRUTURAH,CDCPFESTRUTURA,DTBASESINDICAL,CDSINDICAL,CDTERCEIRO,NRCLASSTRIBUT,VRFAP,NRTIPOTRIBUT,VRALIQDESON,CDFEDERALESTRUT,IDATIVO,NRPARCREGAMB,NRPARCMONBIO,DTINIRESAMB,DTFIMRESAMB,NRTPOBRIGAPRENDIZ,NRTPOBRIGAPCD,CDCAEPFESTRUT,CODCONVENIO,NRCDPROCESSORAT,VRPERCENTRAT,NRCDPROCESSOFAP,VRPERCENTFAP,NRCDCONVENIOTERCEIRO,NRPROCESSOTERCEIRO,NRCDCONTRATAPRENDIZ,IDENTIDSFIMLUCRATIVO,NRCDPROCESSOCONTRATPCD,NRPARCRESPINFADIC,IDENVFAPESO)
            values ({$DEFAULT_PK}-2,{$DEFAULT_ORG},{$DEFAULT_PK}-2,to_date('01/01/2023 00:00:00','DD/MM/RRRR HH24:MI:SS'),null,{$DEFAULT_PK},null,null,'ESTRUTURA TESTES INTEGRACAO 3',null,null,null,null,null,'N','N',{$DEFAULT_PK},null,null,null,to_date('01/01/2023 10:53:53','DD/MM/RRRR HH24:MI:SS'),null,null,${DEFAULT_ORG},null,null,null,null,'ESTRUTURA TESTES INTEGRACAO','ESTRUTURA TESTES INTEGRACAO',null,null,null,null,null,null,null,null,null,'S',null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,'N')",
        ];

        foreach ($sqls as $sql) {
            Util::executeStatementOnConnection($sql);
        }
    }

    public static function destroyTestData(): void {
        $DEFAULT_PK = Util::DEFAULT_PK;
        $DEFAULT_ORG = Util::DEFAULT_ORG;

        $connection = Util::getConnection();
        $connection->setAutoCommit(false);

        $sqls = [
            "Delete From ESTRUTURAH Where NRORG = {$DEFAULT_ORG} And NRESTRUTURAM In ({$DEFAULT_PK},{$DEFAULT_PK}-1)",
            "Delete From ESTRUTURAM Where NRORG = {$DEFAULT_ORG} And NRESTRUTURAM In ({$DEFAULT_PK},{$DEFAULT_PK}-1)",
            "Delete From PARCNEGOCIO Where NRORG = {$DEFAULT_ORG} And NRPARCNEGOCIO In ({$DEFAULT_PK},{$DEFAULT_PK}-1)",
        ];

        foreach ($sqls as $sql) {
            $connection->executeStatement($sql);
        }

        $connection->commit();
    }
}
