/* Sistemas Integrados TEKNISA
   Gesto Consolidada

Nome: 0015_STPTEKO_ETL_ATENDTEK.sql
                                                                            
Descrio:                                                                 
Procedure para preencher os fatos da tabela ETLATENDTEK a partir dos dados 
apurados dos atendimentos registrados no sistema interno Teknisa.

Criao:                                                                   
25/02/2007 - Raphael Ernani

*/

create or replace
PROCEDURE ETL_ATENDTEK
                     (P_DTINI  IN DATE,
                      P_DTFINA IN DATE)
IS
  V_DTFIM           DATE;
  V_DTINI           DATE;
  V_NRSEQFATATEND   VARCHAR2(12);
  V_NRSEQCONATEND   VARCHAR2(12);
  V_DTMES           VARCHAR2(07);
  V_DTANO           VARCHAR2(04);
  V_BIM             VARCHAR2(6);
  V_TRI             VARCHAR2(6);
  V_SEM             VARCHAR2(6);
  V_SITUACAO        VARCHAR2(12);
  V_NEXT_HOUR       VARCHAR2(20);
  V_COUNT           INTEGER := 0;
  /*CONSTANTES*/
  CON_CONCLUIDO     VARCHAR2(9)  := 'Concludo';
  CON_ANDAMENTO     VARCHAR2(12) := 'Em Andamento';
BEGIN
    V_DTINI  := P_DTINI;
    V_DTMES  :=  TO_CHAR(V_DTINI,'MM');
    V_DTANO  :=  TO_CHAR(V_DTINI,'YYYY');
/*APAGA OS DADOS J EXISTENTES A PARTIR DA DATA INCIO*/
    DELETE FROM ETLATENDTEK
    WHERE (ANFATATEND > V_DTANO) OR
          (ANFATATEND = V_DTANO
          AND  MEFATATEND >= V_DTMES);
    DELETE FROM ETLCONATETEK 
    WHERE (ANCONATEND > V_DTANO) OR
          (ANCONATEND = V_DTANO
          AND  MECONATEND >= V_DTMES); 
    COMMIT;
    
    BEGIN
       SELECT NVL(MAX(LPAD(NRSEQFATATEN,12,0)),0)
       INTO   V_NRSEQFATATEND
       FROM   ETLATENDTEK;
    EXCEPTION
    WHEN NO_DATA_FOUND THEN
       V_NRSEQFATATEND := '0';
    END;
    
    BEGIN
       SELECT NVL(MAX(LPAD(NRSEQCONATEN,12,0)),0)
       INTO   V_NRSEQCONATEND
       FROM   ETLCONATETEK;
    EXCEPTION
    WHEN NO_DATA_FOUND THEN
       V_NRSEQCONATEND := '0';
    END;

    WHILE (V_DTINI  <=  P_DTFINA) LOOP

       V_DTFIM := LAST_DAY( V_DTINI);

       V_DTMES  :=  TO_CHAR(V_DTFIM,'MM');
       V_DTANO  :=  TO_CHAR(V_DTFIM,'YYYY');
       V_BIM    :=  TO_CHAR(TRUNC((V_DTMES - 1) / 2) + 1)||'Bim';
       V_TRI    :=  TO_CHAR(TRUNC((V_DTMES - 1) / 3) + 1)||'Tri';
       V_SEM    :=  TO_CHAR(TRUNC((V_DTMES - 1) / 6) + 1)||'Sem';


       FOR C_ETL IN
         (SELECT
                COUNT(DISTINCT PLA.CDFILIAL||PLA.NRTAREFA||PLA.NRPLATIVTARE) AS QTATIVIDADES,
                SUM(APR.DTFAPATIVTAR - APR.DTIAPATIVTAR)*24*60*60 AS QTTMPATIV ,PLA.CDOPERRESP,
                TAR.CDASSUNTO,TAR.CDPRODCOMP,PLA.NMINTATIVTAR,
                PLA.CDFORMACOMU,TAR.CDFILIAL, TAR.CDCLIENTE, TAR.IDSITTAREFA,
                TO_CHAR(PLA.DTINCATIVTAR,'HH24') AS HRDIA
          FROM
                TAREFA TAR,PLANATIVTARE PLA, APROATIVTARE APR
          WHERE
                PLA.CDFILIAL     (+)= TAR.CDFILIAL
            AND PLA.NRTAREFA     (+)= TAR.NRTAREFA
            AND APR.CDFILIAL     (+)= PLA.CDFILIAL
            AND APR.NRTAREFA     (+)= PLA.NRTAREFA
            AND APR.NRPLATIVTARE (+)= PLA.NRPLATIVTARE
            AND TAR.IDORIGTARE      = '2'
            AND TRUNC(PLA.DTINCATIVTAR) BETWEEN V_DTINI AND V_DTFIM
          GROUP BY
                PLA.CDOPERRESP, TAR.CDASSUNTO,TAR.CDPRODCOMP,PLA.NMINTATIVTAR,
                PLA.CDFORMACOMU,TAR.CDFILIAL, TAR.CDCLIENTE, TAR.IDSITTAREFA,
                TO_CHAR(PLA.DTINCATIVTAR,'HH24')
          )
       LOOP
            V_NEXT_HOUR := LPAD(TO_CHAR(TO_NUMBER(C_ETL.HRDIA) + 1),2,'0');
            IF V_NEXT_HOUR > 23 THEN
               V_NEXT_HOUR := '00';
            END IF;
            
            
            IF C_ETL.IDSITTAREFA = 'C' THEN
               V_SITUACAO := CON_CONCLUIDO;
            ELSE
               V_SITUACAO := CON_ANDAMENTO;
            END IF;

            V_NRSEQFATATEND := LPAD(V_NRSEQFATATEND + 1, 12 ,'0');

            INSERT INTO ETLATENDTEK
                  (
                   NRSEQFATATEN,    ANFATATEND,         MEFATATEND,       BIFATATEND,
                   TRFATATEND,      SEFATATEND,         CDOPERADOR,       CDASSUNTO,
                   CDPRODUTO,       CDFORMACOMU,        CDCLIENTE,        NMINTATIVTAR,
                   QTATEATEND,      QTTMPATEND,         CDFILIAL,         IDSTATUSATEND,
                   HRDIAATEND
                  )
            VALUES
                  (
                  V_NRSEQFATATEND,  V_DTANO,            V_DTMES,          V_BIM,
                  V_TRI,            V_SEM,              C_ETL.CDOPERRESP, C_ETL.CDASSUNTO,
                  C_ETL.CDPRODCOMP, C_ETL.CDFORMACOMU,  C_ETL.CDCLIENTE,  C_ETL.NMINTATIVTAR,
                  C_ETL.QTATIVIDADES, C_ETL.QTTMPATIV,  C_ETL.CDFILIAL,   V_SITUACAO,
                  C_ETL.HRDIA||':00 - '||V_NEXT_HOUR||':00'
                  );
                  
            V_COUNT := V_COUNT + 1;
            IF V_COUNT >= 500 THEN
               COMMIT;
               V_COUNT := 0;
            END IF;

       END LOOP;
       
       COMMIT;
       
       FOR C_CLIENTE IN
          (SELECT CDCLIENTE
           FROM   CLIENTE
           WHERE  CDCLIENTE NOT IN( SELECT CDCLIENTE 
                                    FROM ETLATENDTEK
                                    WHERE ANFATATEND = V_DTANO
                                      AND MEFATATEND = MEFATATEND)
          
          )
       LOOP
            V_NRSEQFATATEND := LPAD(V_NRSEQFATATEND + 1, 12 ,'0');
            INSERT INTO ETLATENDTEK
                  (
                   NRSEQFATATEN,    ANFATATEND,         MEFATATEND,       BIFATATEND,
                   TRFATATEND,      SEFATATEND,         CDCLIENTE,
                   QTATEATEND,      QTTMPATEND
                  )
            VALUES
                  (
                  V_NRSEQFATATEND,  V_DTANO,            V_DTMES,          V_BIM,
                  V_TRI,            V_SEM,              C_CLIENTE.CDCLIENTE,
                  0,                0 
                  ); 
                  
            V_COUNT := V_COUNT + 1;
            IF V_COUNT >= 500 THEN
               COMMIT;
               V_COUNT := 0;
            END IF;                  
       END LOOP;
       COMMIT;
       
       

       FOR C_ETL IN
               (SELECT 
                      COUNT(DISTINCT TAR.CDFILIAL||TAR.NRTAREFA) AS QTTAREFAS,
                      SUM(TEMPO_CONC_ATEND(TAR.CDFILIAL, TAR.NRTAREFA)*3600) AS TEMPOCONC,
                      SUM(TEMPRESPATEN)*3600 AS TEMPRESPATEN,
                      SUM(QTRESP) AS QTRESP,
                      CDOPERRESP, CDASSUNTO, CDPRODCOMP, TAR.CDFILIAL, 
                      CDCLIENTE, IDSITTAREFA
                FROM 
                    TAREFA TAR,
                    (SELECT CDFILIAL, NRTAREFA, COUNT(*) AS QTRESP,
                            TEMPO_RESP_ATEND(CDFILIAL, NRTAREFA) as TEMPRESPATEN
                     FROM TAREFA
                     WHERE TEMPO_RESP_ATEND(CDFILIAL, NRTAREFA) IS NOT NULL
                       AND IDORIGTARE  = '2'
                     GROUP BY CDFILIAL,NRTAREFA,TEMPO_RESP_ATEND(CDFILIAL, NRTAREFA))  TR
                WHERE 
                      TR.CDFILIAL(+)= TAR.CDFILIAL
                  AND TR.NRTAREFA(+)= TAR.NRTAREFA                  
                  AND TAR.CDFILIAL||TAR.NRTAREFA IN( SELECT CDFILIAL||NRTAREFA 
                                                     FROM PLANATIVTARE 
                                                     WHERE TRUNC(DTINCATIVTAR) BETWEEN V_DTINI AND V_DTFIM )
                  AND (IDORIGTARE  = '2') 
                  AND (IDSITTAREFA = 'C')
                GROUP BY
                      CDOPERRESP, CDASSUNTO, CDPRODCOMP, TAR.CDFILIAL, 
                      CDCLIENTE, IDSITTAREFA
                      
               UNION
               
               SELECT 
                      COUNT(DISTINCT TAR.CDFILIAL||TAR.NRTAREFA) AS QTTAREFAS,
                      NULL AS TEMPOCONC,
                      SUM(TEMPRESPATEN)*3600 AS TEMPRESPATEN,
                      SUM(QTRESP) AS QTRESP,
                      CDOPERRESP, CDASSUNTO, CDPRODCOMP, TAR.CDFILIAL, 
                      CDCLIENTE, IDSITTAREFA
                FROM 
                    TAREFA TAR,
                    (SELECT CDFILIAL, NRTAREFA, COUNT(*) AS QTRESP,
                            TEMPO_RESP_ATEND(CDFILIAL, NRTAREFA) as TEMPRESPATEN
                     FROM TAREFA
                     WHERE TEMPO_RESP_ATEND(CDFILIAL, NRTAREFA) IS NOT NULL
                       AND IDORIGTARE  = '2'
                     GROUP BY CDFILIAL,NRTAREFA,TEMPO_RESP_ATEND(CDFILIAL, NRTAREFA))  TR
                WHERE 
                      TR.CDFILIAL(+)= TAR.CDFILIAL
                  AND TR.NRTAREFA(+)= TAR.NRTAREFA
                  AND TAR.CDFILIAL||TAR.NRTAREFA IN( SELECT CDFILIAL||NRTAREFA 
                                                     FROM PLANATIVTARE 
                                                     WHERE TRUNC(DTINCATIVTAR) BETWEEN V_DTINI AND V_DTFIM )
                  AND (IDORIGTARE  = '2') 
                  AND (IDSITTAREFA <> 'C')
                GROUP BY
                      CDOPERRESP, CDASSUNTO, CDPRODCOMP, TAR.CDFILIAL, 
                      CDCLIENTE, IDSITTAREFA
               )
       LOOP
            IF C_ETL.IDSITTAREFA = 'C' THEN
               V_SITUACAO := CON_CONCLUIDO;
            ELSE
               V_SITUACAO := CON_ANDAMENTO;
            END IF;

            V_NRSEQCONATEND := LPAD(V_NRSEQCONATEND + 1, 12 ,'0');      
      
            INSERT INTO ETLCONATETEK
                  (
                   NRSEQCONATEN,    ANCONATEND,         MECONATEND,       BICONATEND,
                   TRCONATEND,      SECONATEND,         CDOPERADOR,       CDASSUNTO,
                   CDPRODUTO,       CDCLIENTE,          QTCONATEND,       QTTMPCONATEN,
                   CDFILIAL,        IDSTATUSATEND,      QTTMPRESATEN,     QTRESATEND 
                  )
            VALUES
                  (
                  V_NRSEQCONATEND,  V_DTANO,            V_DTMES,            V_BIM,
                  V_TRI,            V_SEM,              C_ETL.CDOPERRESP,   C_ETL.CDASSUNTO,
                  C_ETL.CDPRODCOMP, C_ETL.CDCLIENTE,    C_ETL.QTTAREFAS,    C_ETL.TEMPOCONC,  
                  C_ETL.CDFILIAL,   V_SITUACAO,         C_ETL.TEMPRESPATEN, C_ETL.QTRESP
                  );
                  
            V_COUNT := V_COUNT + 1;
            IF V_COUNT >= 500 THEN
               COMMIT;
               V_COUNT := 0;
            END IF;      

       END LOOP;

       COMMIT;
       V_DTINI := ADD_MONTHS(V_DTINI,1);
    END LOOP;
END;
/


DECLARE
    V_CDSEQUENCIAL VARCHAR2(10) NOT NULL := 0;
BEGIN
    /*REGISTRANDO A PROCEDURE PARA O AGENDADOR DE EVENTOS*/
    SELECT COUNT(*)
    INTO   V_CDSEQUENCIAL
    FROM   EVENTOAEV 
    WHERE  NMOBJBDEVEN = 'ETL_ATENDTEK';
    
    IF V_CDSEQUENCIAL = 0 THEN     --> S ADICIONA A PROCEDURE SE ELA NO EXISTIR AINDA NO CADASTRO
        BEGIN
            SELECT LPAD(NVL(MAX(CDEVENTOAEV)+1,0),10,'0')
            INTO   V_CDSEQUENCIAL
            FROM   EVENTOAEV;    

            INSERT INTO EVENTOAEV 
            (CDEVENTOAEV, NMEVENTOAEV, IDTPEVENAEV, CDOPERINCL, DTINCLEVEN, NMOBJBDEVEN)
            VALUES
            (V_CDSEQUENCIAL,'Resultados Atendimento Teknisa','STP',NULL,SYSDATE,'ETL_ATENDTEK');

            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTINI','D','E','Data Inicial','N',0,'01');

            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTFINA','D','E','Data Final','N',0,'02');

            COMMIT;
        EXCEPTION
           WHEN OTHERS THEN
              RAISE_APPLICATION_ERROR (-20001,'Falha no cadastro de eventos agendveis.');        
        END;
    END IF;
END;
/

/*Cadastrando os fatos e dimenses para o DW*/
BEGIN
   CRIA_FATOS('ETLATENDTEK', 'QTATEATEND', 'Qtd. Evol. Atendimentos',     'Suporte');
   CRIA_FATOS('ETLATENDTEK', 'QTTMPATEND', 'Qtd. de Tempo das Evolues', 'Suporte');
  
   CRIA_DIME('ETLATENDTEK',  'CDFILIAL',      'Filial',                 'FILIAL',    'CDFILIAL',    'NMFILIAL',    NULL); 
   CRIA_DIME('ETLATENDTEK',  'CDASSUNTO',     'Assunto',                'ASSUNTO',   'CDASSUNTO',   'NMASSUNTO',   NULL); 
   CRIA_DIME('ETLATENDTEK',  'CDFORMACOMU',   'Forma de Comunicao',   'FORMACOMU', 'CDFORMACOMU', 'NMFORMACOMU', NULL);    
   CRIA_DIME('ETLATENDTEK',  'CDCLIENTE',     'Cliente',                'CLIENTE',   'CDCLIENTE',   'NMFANTCLIE',  NULL);    
   CRIA_DIME('ETLATENDTEK',  'CDOPERADOR',    'Operador',               'OPERADOR',  'CDOPERADOR',  'NMOPERADOR',  NULL);    
   CRIA_DIME('ETLATENDTEK',  'CDPRODUTO',     'Produto',                'PRODUTO',   'CDPRODUTO',   'NMPRODUTO',   NULL);    
   CRIA_DIME('ETLATENDTEK',  'IDSTATUSATEND', 'Status do Atendimento',   NULL,        NULL,          NULL,         NULL);     
   CRIA_DIME('ETLATENDTEK',  'NMINTATIVTAR',  'Interlocutor',            NULL,        NULL,          NULL,         NULL); 
   CRIA_DIME('ETLATENDTEK',  'ANFATATEND',    'Ano',                     NULL,        NULL,          NULL,         NULL);    
   CRIA_DIME('ETLATENDTEK',  'MEFATATEND',    'Ms',                     NULL,        NULL,          NULL,         NULL);    
   CRIA_DIME('ETLATENDTEK',  'BIFATATEND',    'Bimestre',                NULL,        NULL,          NULL,         NULL);    
   CRIA_DIME('ETLATENDTEK',  'TRFATATEND',    'Trimestre',               NULL,        NULL,          NULL,         NULL);    
   CRIA_DIME('ETLATENDTEK',  'SEFATATEND',    'Semestre',                NULL,        NULL,          NULL,         NULL);    


   CRIA_FATOS('ETLCONATETEK', 'QTCONATEND',   'Qtd. de Atendimentos',          'Suporte');
   CRIA_FATOS('ETLCONATETEK', 'QTRESATEND',   'Qtd. Atendimentos Respondidos', 'Suporte');
   CRIA_FATOS('ETLCONATETEK', 'QTTMPCONATEN', 'Tempo de Conc. dos Atend.',     'Suporte');
   CRIA_FATOS('ETLCONATETEK', 'QTTMPRESATEN', 'Tempo de Resp. dos Atend.',     'Suporte');
  
   CRIA_DIME('ETLCONATETEK',  'CDFILIAL',      'Filial',                 'FILIAL',    'CDFILIAL',    'NMFILIAL',    NULL); 
   CRIA_DIME('ETLCONATETEK',  'CDASSUNTO',     'Assunto',                'ASSUNTO',   'CDASSUNTO',   'NMASSUNTO',   NULL); 
   CRIA_DIME('ETLCONATETEK',  'CDCLIENTE',     'Cliente',                'CLIENTE',   'CDCLIENTE',   'NMFANTCLIE',  NULL);    
   CRIA_DIME('ETLCONATETEK',  'CDOPERADOR',    'Operador',               'OPERADOR',  'CDOPERADOR',  'NMOPERADOR',  NULL);    
   CRIA_DIME('ETLCONATETEK',  'CDPRODUTO',     'Produto',                'PRODUTO',   'CDPRODUTO',   'NMPRODUTO',   NULL);    
   CRIA_DIME('ETLCONATETEK',  'IDSTATUSATEND', 'Status do Atendimento',   NULL,        NULL,          NULL,         NULL);     
   CRIA_DIME('ETLCONATETEK',  'ANCONATEND',    'Ano',                     NULL,        NULL,          NULL,         NULL);    
   CRIA_DIME('ETLCONATETEK',  'MECONATEND',    'Ms',                     NULL,        NULL,          NULL,         NULL);    
   CRIA_DIME('ETLCONATETEK',  'BICONATEND',    'Bimestre',                NULL,        NULL,          NULL,         NULL);    
   CRIA_DIME('ETLCONATETEK',  'TRCONATEND',    'Trimestre',               NULL,        NULL,          NULL,         NULL);    
   CRIA_DIME('ETLCONATETEK',  'SECONATEND',    'Semestre',                NULL,        NULL,          NULL,         NULL);    

   COMMIT; 
END;
/



-- VERSAO --
BEGIN ATUVEROBJ('ETL_ATENDTEK','5.01.000'); END;
/